package consolerunner;

import core.*;
import java.util.*;

import samples.*;
import javax.xml.ws.soap.SOAPFaultException;

public class Program {

	public static void main(String[] args) throws Throwable {
		// REMEMBER to remove ID and Password before desseminating
		System.out
				.println("Thomson Reuters DataScope Select API Samples Console Runner");
		System.out.println();

		// obtain all samples
		Class<?>[] samplelist = JavaUtility.getClasses("samples");

		// code for console menu
		int currmenulevel = 1;
		String currdirectory = "samples";
		Stack<String> directorypaths = new Stack<String>();
		boolean exitflag = false;
		String classfullname = "";
		int ichoice;
		ArrayList<String> displaylist;
		ArrayList<String> fulllist = CoreUtility.getfulllist(samplelist);
		ArrayList<String> samples = new ArrayList<String>();

		while (!exitflag) {
			displaylist = CoreUtility.getdisplaylist(samplelist, currmenulevel,
					currdirectory, directorypaths);
			System.out.print("Current path : ");
			for (int k = 0; k < directorypaths.size(); k++) {
				System.out.print(directorypaths.get(k).toString() + "->");
			}
			System.out.println(currdirectory);
			System.out
					.println("Please select categories/sample using linenumber and press <Enter>: ");
			for (int m = 0; m < displaylist.size(); m++) {
				System.out.println(Integer.toString(m) + " "
						+ displaylist.get(m).toString());
			}
			System.out.println("Press 'U' and <Enter> to go back.");
			System.out.println("Press 'A' to run all samples.");
			System.out.println("Please 'E' and <Enter> to exit.");
			Scanner scan = new Scanner(System.in);
			if (scan.hasNextInt()) {
				ichoice = scan.nextInt();
				if (ichoice < 0 || ichoice > displaylist.size() - 1) {
					System.out
							.println("Your choice is out of range. Please try again");
				} else {
					if (ichoice >= 0 && ichoice < displaylist.size()) {
						// Verify if selected is a class and run it.
						classfullname = "";
						for (int j = 0; j < directorypaths.size(); j++) {
							classfullname = classfullname
									+ directorypaths.get(j) + ".";
						}
						classfullname = classfullname + currdirectory + "."
								+ displaylist.get(ichoice).toString();

						if (fulllist.contains(classfullname)) {
							// Run this class
							samples.clear();
							samples.add(classfullname);
							runsample(samples);
						} else {
							// move to select category or class
							currmenulevel++;
							directorypaths.push(currdirectory);
							currdirectory = displaylist.get(ichoice).toString();
						}
					}
				}
			} else if (scan.hasNext("U") || scan.hasNext("u")) {
				if (currmenulevel > 1) {
					// move up one level to previous directory
					currdirectory = directorypaths.pop().toString();
					currmenulevel--;
				} else {
					System.out
							.println("Already at the top level. Cannot go back up. Please try again.");
				}
			} else if (scan.hasNext("A") || scan.hasNext("a")) {
				// Run all samples
				runsample(fulllist);
			} else if (scan.hasNext("E") || scan.hasNext("e")) {
				exitflag = true;
			} else {
				System.out.println("Your input is invalid. Please try again.");
				Thread.sleep(2000); // pause before displaying list again
			}
		}
		System.out.println("Exiting");
	}

	public static void runsample(ArrayList<String> samplelist) throws Throwable {
		for (int i = 0; i < samplelist.size(); i++) {
			try {
				SampleRunner sample = new SampleRunner(samplelist.get(i)
						.toString());
				sample.Run();
			} catch (ClientErrorException ex) {
				System.out.println();
				System.out.println(ex.getMessage());
				System.out.println("CLIENT ERROR:");
				System.out.println("  " + ex.getClientError());
				System.out.println("  Messages =");
				for (int j = 0; j < ex.getClientErrorMessage().getString()
						.size(); j++) {
					System.out.println("  "
							+ ex.getClientErrorMessage().getString().get(j));
				}
			} catch (ServerErrorException ex) {
				System.out.println();
				System.out.println("SERVER ERROR:");
				System.out.println("  " + ex.getErrorDetail());
				System.out.println("  ErrorInstanceId = "
						+ ex.getErrorInstanceId());
				System.out.println("  Host = " + ex.getHost());
			} catch (InvalidOutputFieldsException ex) {
				System.out.println();
				System.out.println("INVALID OUTPUT FIELDS:");
				System.out.println("  " + ex.getError());
				for (int j = 0; j < ex.getInvalidfields().getString().size(); j++) {
					System.out.println("  "
							+ ex.getInvalidfields().getString().get(j));
				}
			} catch (SOAPFaultException ex) {
				System.out.println();
				System.out.println("GENERAL SOAP FAULT:");
				System.out.println("  " + ex.getMessage());
			} catch (MaxRetriesException ex) {
				System.out.println();
				System.out.println("RETRIES FAULT:");
				System.out.println("  " + ex.getMessage() + "  "
						+ Integer.toString(ex.getRetries()) + " times ");
			} catch (Exception ex) {
				System.out.println("ALL ERROR");
				ex.printStackTrace();
			}
			Thread.sleep(1000);
		}

	}
}
