package core;

import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.Stack;

// TODO: Auto-generated Javadoc
/**
 * The Class CoreUtility discovers all the code samples available to the host
 * application..
 */
public class CoreUtility {

	/**
	 * Gets the exclusion classes.
	 * 
	 * @return the exclusion
	 */
	public static ArrayList<String> getexclusion() {
		ArrayList<String> excludelist = new ArrayList<String>();
		excludelist.add("ApiUtility");
		excludelist.add("ClientErrorException");
		excludelist.add("ExtractionUtility");
		excludelist.add("InvalidOutputFieldsException");
		excludelist.add("JavaUtility");
		excludelist.add("ServerErrorException");
		excludelist.add("MaxRetriesException");
		return excludelist;
	}

	/**
	 * Gets the full list of samples that can be run.
	 * 
	 * @param samplelist
	 *            the samplelist
	 * @return the full list
	 */
	public static ArrayList<String> getfulllist(Class<?>[] samplelist) {

		ArrayList<String> mySamples = new ArrayList<String>();
		ArrayList<String> excludelist = getexclusion();
		String lastelement = "";

		for (int i = 0; i < samplelist.length; i++) {
			StringTokenizer Tok = new StringTokenizer(samplelist[i].getName(),
					".");
			while (Tok.hasMoreElements()) {
				lastelement = Tok.nextElement().toString();
			}
			if (!excludelist.contains(lastelement)) {
				mySamples.add(samplelist[i].getName());
			}
		}
		return mySamples;
	}

	/**
	 * Gets the display list at the current hierarchy level being selected. This
	 * is used for the console menu.
	 * 
	 * @param samplelist
	 *            the sample list
	 * @param currmenulevel
	 *            the curr menu level
	 * @param currdirectory
	 *            the current directory
	 * @param directorypaths
	 *            the directory paths
	 * @return the display list
	 */
	public static ArrayList<String> getdisplaylist(Class<?>[] samplelist,
			int currmenulevel, String currdirectory, Stack<?> directorypaths) {
		// This method will return the list of categories or class names
		ArrayList<String> displaylist = new ArrayList<String>();
		ArrayList<String> excludelist = getexclusion();
		String tempdirectory = "";
		String elementname = "";
		String fulldirectorypath = "";

		for (int j = 0; j < directorypaths.size(); j++) {
			fulldirectorypath = fulldirectorypath
					+ directorypaths.get(j).toString() + ".";
		}
		fulldirectorypath = fulldirectorypath + currdirectory + ".";

		for (int i = 0; i < samplelist.length; i++) {

			StringTokenizer Tok = new StringTokenizer(samplelist[i].getName(),
					".");
			int level = 0;
			while (Tok.hasMoreElements()) {
				if (level == currmenulevel
						&& tempdirectory.equals(currdirectory)
						&& (samplelist[i].getName()
								.startsWith(fulldirectorypath))) {
					elementname = Tok.nextElement().toString();
					if (!displaylist.contains(elementname)
							&& (!excludelist.contains(elementname))) {
						displaylist.add(elementname);
					}
				} else
					tempdirectory = Tok.nextElement().toString();
				level++;
				if (level > currmenulevel) {
					break;
				}
			}
		}
		return displaylist;
	}
}
