package samples;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;

import com.reuters.datascopeselect.extractionservice.v1.*;

import javax.xml.soap.SOAPFault;
import javax.xml.ws.Holder;
import javax.xml.ws.soap.SOAPFaultException;
import org.w3c.dom.NodeList;

// TODO: Auto-generated Javadoc
/**
 * The Class ApiUtility contains methods for performing common tasks with the
 * API.
 */
public class ApiUtility {

	private static Properties propertyBag;

	public static Properties getPropertyBag() throws FileNotFoundException,
			IOException {
		if (propertyBag == null) {	
			FileInputStream inputStream = new FileInputStream(new File("api_config.properties"));
			propertyBag = new Properties();
			propertyBag.load(inputStream);
		}

		return propertyBag;
	}

	/**
	 * Creates the credentials.
	 * 
	 * @return the credentials header
	 * @throws IOException
	 * @throws FileNotFoundException
	 */
	public static CredentialsHeader CreateCredentials()
			throws FileNotFoundException, IOException {
		CredentialsHeader tmpch = new CredentialsHeader();

		// Please provide your account username and password
		tmpch.setUsername(getPropertyBag().getProperty("api.username"));
		tmpch.setPassword(getPropertyBag().getProperty("api.password"));
		return tmpch;
	}

	/**
	 * Convert fault exception. Examines the SOAP fault returned by the API and
	 * converts its contents into client error if a client error occurred,
	 * server error if a server error occurred, if any other error occurred.
	 * containing the SOAP fault.
	 * 
	 * @param ex
	 *            the ex
	 * @return the exception
	 */
	public static Exception ConvertFaultException(SOAPFaultException ex) {
		SOAPFault fault = ex.getFault();
		if (fault.hasDetail()) {

			try {
				String faultCode = fault.getFaultCode();
				if (faultCode.equals("soap:Client")) {
					ClientError clientError = ClientErrorDeserializer(fault);
					return new ClientErrorException(ex.getMessage(),
							clientError);
				}

				if (faultCode.equals("soap:Server")) {
					ServerError serverError = ServerErrorDeserializer(fault);
					return new ServerErrorException(ex.getMessage(),
							serverError);
				}
			} catch (Exception e) {
				// swallow any failed attempt to convert a SOAP fault into a
				// ClientError or ServerError
				// and revert to just returning SOAPFaultException
			}
		}
		return ex;
	}

	/**
	 * Client error deserializer.
	 * 
	 * @param fault
	 *            the fault
	 * @return the client error
	 */
	public static ClientError ClientErrorDeserializer(SOAPFault fault) {
		ClientError ce = new ClientError();
		ArrayOfString messages = new ArrayOfString();

		NodeList SOAPNodes = fault.getDetail().getFirstChild().getChildNodes();
		for (int i = 0; i < SOAPNodes.getLength(); i++) {
			if (SOAPNodes.item(i).getNodeName().equals("ErrorCode")) {
				ce.setErrorCode(SOAPNodes.item(i).getTextContent());
			}
			if (SOAPNodes.item(i).getNodeName().equals("Messages")) {
				messages.getString().add(SOAPNodes.item(i).getTextContent());
			}
		}
		ce.setMessages(messages);
		return ce;
	}

	/**
	 * Server error deserializer.
	 * 
	 * @param fault
	 *            the fault
	 * @return the server error
	 */
	public static ServerError ServerErrorDeserializer(SOAPFault fault) {
		ServerError se = new ServerError();
		NodeList SOAPNodes = fault.getDetail().getFirstChild().getChildNodes();
		for (int i = 0; i < SOAPNodes.getLength(); i++) {
			if (SOAPNodes.item(i).getNodeName().equals("Host")) {
				se.setHost(SOAPNodes.item(i).getTextContent());
			}
			if (SOAPNodes.item(i).getNodeName().equals("ErrorInstanceId")) {
				se.setErrorInstanceId(SOAPNodes.item(i).getTextContent());
			}
			if (SOAPNodes.item(i).getNodeName().equals("ErrorDetail")) {
				se.setErrorDetail(SOAPNodes.item(i).getTextContent());
			}
		}
		return se;
	}

	/**
	 * Execute.
	 * 
	 * @param <U>
	 *            the generic type
	 * @param webapi
	 *            the web api
	 * @return the u instance
	 * @throws MaxRetriesException
	 *             the max retries exception
	 */
	@SuppressWarnings("unchecked")
	public static <U> U Execute(String webapi) throws MaxRetriesException {
		int MaxRetries = 3;
		int retries = 0;
		while (retries < MaxRetries) {
			retries++;
			try {
				String methodName = webapi;
				CredentialsHeader credentials = ApiUtility.CreateCredentials();
				Holder<CredentialsHeader> hch = ApiUtility
						.createhch(credentials);
				Holder<OperationInfoHeader> hoih = ApiUtility.createhoih();

				com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service service = new com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service();
				com.reuters.datascopeselect.extractionservice.v1.ExtractionService proxy = service
						.getExtractionService();
				Class<?> cl = Class
						.forName("com.reuters.datascopeselect.extractionservice.v1.ExtractionService");

				Class<?> arguments[] = new Class[] { hch.getClass(),
						EjvServerHeader.class, hoih.getClass() };

				java.lang.reflect.Method objMethod = cl.getMethod(methodName,
						arguments);

				Object result = objMethod.invoke(proxy, hch, null, hoih);
				return (U) result;
			} catch (Exception e) {
			}
		}
		if (retries >= MaxRetries) {
			throw new MaxRetriesException(
					"The maximum number of configured retries has been exceeded.",
					retries);
		}
		return null;
	}

	/**
	 * Createhch.
	 * 
	 * @param ch
	 *            the credentials header
	 * @return the holder
	 */
	public static Holder<CredentialsHeader> createhch(CredentialsHeader ch) {
		Holder<CredentialsHeader> hch = new Holder<CredentialsHeader>();
		hch.value = ch;
		return hch;
	}

	/**
	 * Createhoih.
	 * 
	 * @return the operation info holder
	 */
	public static Holder<OperationInfoHeader> createhoih() {
		Holder<OperationInfoHeader> hoih = new Holder<OperationInfoHeader>();
		hoih.value = new OperationInfoHeader();
		return hoih;
	}
}
