package samples.datadictionary;

import javax.xml.ws.Holder;
import javax.xml.ws.soap.SOAPFaultException;

import samples.ApiUtility;
import samples.ClientErrorException;
import samples.ServerErrorException;

import com.reuters.datascopeselect.extractionservice.v1.*;
import core.ISample;

public class CoraxSample implements ISample {

	public void Execute() throws ClientErrorException, ServerErrorException,
			Exception {
		CredentialsHeader credentials = ApiUtility.CreateCredentials();
		Holder<CredentialsHeader> hch = ApiUtility.createhch(credentials);
		Holder<OperationInfoHeader> hoih = ApiUtility.createhoih();

		com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service service = new com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service();
		com.reuters.datascopeselect.extractionservice.v1.ExtractionService proxy = service
				.getExtractionService();

		String capitalChangeEvent = "CAP";
		String dividentEvent = "DIV";

		System.out.println("Getting all Corporate Action fields that:");
		System.out.println("- are associated with Capital Change "
				+ capitalChangeEvent + " and Dividend " + dividentEvent
				+ " events");

		ArrayOfString valcorax = new ArrayOfString();
		valcorax.getString().add(capitalChangeEvent);
		valcorax.getString().add(dividentEvent);
		DataDictionaryRequestCorax request = new DataDictionaryRequestCorax();
		request.setCoraxEvents(valcorax);

		DataDictionaryResponse response = null;
		try {
			response = proxy.define(request, hch, null, hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}

		System.out.println(response.getFields().getDataDictionaryField().size()
				+ " fields found:");
		for (int i = 0; i < response.getFields().getDataDictionaryField()
				.size(); i++) {
			System.out.println(response.getFields().getDataDictionaryField()
					.get(i).getName()
					+ ": "
					+ response.getFields().getDataDictionaryField().get(i)
							.getDescription());

			// print out AssetTypes
			System.out.print(" - Asset Types = ");
			for (int j = 0; j < response.getFields().getDataDictionaryField()
					.get(i).getAssetTypes().getString().size() - 1; j++) {
				System.out.print(response.getFields().getDataDictionaryField()
						.get(i).getAssetTypes().getString().get(j)
						+ " ");
			}
			System.out.println("");

			// print out Corax Events
			System.out.print(" - Corax Events = ");
			for (int k = 0; k < response.getFields().getDataDictionaryField()
					.get(i).getCoraxEvents().getString().size() - 1; k++) {
				System.out.print(response.getFields().getDataDictionaryField()
						.get(i).getCoraxEvents().getString().get(k)
						+ " ");
			}
			System.out.println("");

			// print out Bond Schedule Types
			System.out.print(" - Bond Schedule Types = ");
			for (int l = 0; l < response.getFields().getDataDictionaryField()
					.get(i).getBondScheduleTypes().getString().size() - 1; l++) {
				System.out.print(response.getFields().getDataDictionaryField()
						.get(i).getBondScheduleTypes().getString().get(l)
						+ " ");
			}
			System.out.println("");
		}
	}
}
