package samples.extraction.analytics;

import javax.xml.ws.Holder;
import javax.xml.ws.soap.SOAPFaultException;

import com.reuters.datascopeselect.extractionservice.v1.*;

import samples.ApiUtility;
import samples.ClientErrorException;
import samples.ServerErrorException;
import core.ISample;

public class FixedIncomeAnalyticsSample implements ISample {
	public void Execute() throws ClientErrorException, ServerErrorException,
			Exception {
		CredentialsHeader credentials = ApiUtility.CreateCredentials();
		Holder<CredentialsHeader> hch = ApiUtility.createhch(credentials);
		Holder<OperationInfoHeader> hoih = ApiUtility.createhoih();

		com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service service = new com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service();
		com.reuters.datascopeselect.extractionservice.v1.ExtractionService proxy = service
				.getExtractionService();

		InstrumentIdentifier cf1 = new InstrumentIdentifier();
		cf1.setIdentifierType("CSP");
		cf1.setIdentifierValue("911759ED6");

		ArrayOfInstrumentIdentifier cf = new ArrayOfInstrumentIdentifier();
		cf.getInstrumentIdentifier().add(cf1);

		ArrayOfString outputFields = new ArrayOfString();
		outputFields.getString().add("Asset ID");
		outputFields.getString().add("Asset Status");
		outputFields.getString().add("Asset SubType");
		outputFields.getString().add("Asset SubType Description");
		outputFields.getString().add("Asset Type");
		outputFields.getString().add("Asset Type Description");
		outputFields.getString().add("Benchmark");
		outputFields.getString().add("Benchmark Type");
		outputFields.getString().add("Benchmark Value");
		outputFields.getString().add("Bond Structure");
		outputFields.getString().add("Bond Type");
		outputFields.getString().add("Bond Type Description");
		outputFields.getString().add("Econometric Average Life");
		outputFields.getString().add("Econometric Average Life Date");
		outputFields.getString().add("Pricing Method Code");
		outputFields.getString().add("Pricing Method Value");
		outputFields.getString().add("Redemption Date");
		outputFields.getString().add("Redemption Event");
		outputFields.getString().add("Redemption Type");
		outputFields.getString().add("Sink Speed");
		outputFields.getString().add("Years To Redemption");
		outputFields.getString().add("Yield Type");
		outputFields.getString().add("Yield Type Description");

		System.out.println("Performing a Fixed Income Analytics extraction that:");
		System.out.println("- Retrieve user-defined price/yield (P/Y) calculations, duration and convexity calculations, spread calculations, ");
		System.out.println("- and complete cash flow calculations for U.S. government, international, and corporate securities,");
		System.out.println("- mortgage-backed securities (MBS),");
		System.out.println("- collateralized mortgage obligations (CMOs),");
		System.out.println("- asset-backed and whole loan-backed deals, and a wide range of pools.");
		System.out.println("- returns data for the following output fields:");
		for (int n = 0; n < outputFields.getString().size(); n++) {
			System.out.println("  " + outputFields.getString().get(n));
		}

		System.out.println("Validating instruments...");
		InstrumentValidationResponse instrument = new InstrumentValidationResponse();
		InstrumentValidationRequest ivreq = new InstrumentValidationRequest();
		ivreq.setIdentifiers(cf);

		try {
			instrument = proxy.validateInstruments(ivreq, hch, null, hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}

		System.out.println("Performing extraction...");
		ArrayOfValidatedInstrument arrayValidated = new ArrayOfValidatedInstrument();
		arrayValidated = instrument.getInstruments();
		InstrumentExtractionRequestFixedIncomeAnalytics request = new InstrumentExtractionRequestFixedIncomeAnalytics();
		request.setInstruments(arrayValidated);
		request.setOutputFields(outputFields);
		ExtractionResponse response = new ExtractionResponse();

		try {
			response = proxy.extract(request, hch, null, hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}

		System.out.println("Row Count = " + response.getRowCount());
		System.out.println();
		System.out
				.println("Row Data (in CSV format - copy/paste into new file to open in a spreadsheet application) =");
		System.out.println();

		for (int i = 0; i < response.getColumns().getExtractionColumn().size(); i++) {
			System.out.print(response.getColumns().getExtractionColumn().get(i)
					.getName()
					+ ",");
		}
		System.out.println();

		for (int j = 0; j < response.getRowCount(); j++) {
			for (int i = 0; i < response.getColumns().getExtractionColumn()
					.size(); i++) {
				System.out.print(response.getColumns().getExtractionColumn()
						.get(i).getValues().getDoubleOrEmptyOrDate().get(j)
						.toString()
						+ ",");
			}
			System.out.println();
		}

		System.out.println("Extraction Messages:");
		for (int m = 0; m < response.getMessages().getString().size(); m++) {
			System.out.println(response.getMessages().getString().get(m));
		}
		System.out.println("");
	}
}
