package samples.extraction.analytics;

import javax.xml.ws.Holder;
import javax.xml.ws.soap.SOAPFaultException;

import com.reuters.datascopeselect.extractionservice.v1.*;

import samples.ApiUtility;
import samples.ClientErrorException;
import samples.ServerErrorException;
import core.ISample;

public class TechnicalIndicatorAnalyticsSample implements ISample {
	public void Execute() throws ClientErrorException, ServerErrorException,
			Exception {
		CredentialsHeader credentials = ApiUtility.CreateCredentials();
		Holder<CredentialsHeader> hch = ApiUtility.createhch(credentials);
		Holder<OperationInfoHeader> hoih = ApiUtility.createhoih();

		com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service service = new com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service();
		com.reuters.datascopeselect.extractionservice.v1.ExtractionService proxy = service
				.getExtractionService();

		InstrumentIdentifier cf1 = new InstrumentIdentifier();
		cf1.setIdentifierType("RIC");
		cf1.setIdentifierValue("IBM.N");

		ArrayOfInstrumentIdentifier cf = new ArrayOfInstrumentIdentifier();
		cf.getInstrumentIdentifier().add(cf1);

		ArrayOfString outputFields = new ArrayOfString();
		outputFields.getString().add("Alpha - 3 Year Weekly");
		outputFields.getString().add("Alpha - 5 Year Monthly");
		outputFields.getString().add("Beta - 3 Year Weekly");
		outputFields.getString().add("Beta - 5 Year Monthly");
		outputFields.getString().add("Average Volume - 10 Days");
		outputFields.getString().add("Volatility - 10 days");
		outputFields.getString().add("Instrument ID");
		outputFields.getString().add("Instrument ID Type");
		outputFields.getString().add("Asset ID");
		outputFields.getString().add("Asset Type");
		outputFields.getString().add("Asset Type Description");
		outputFields.getString().add("Country of Incorporation");

		System.out.println("Performing a Technical Indicators extraction that:");
		System.out.println("- Retrieve technical analytical data like Alpha, Beta, ");
		System.out.println("- Average Volume, Moving Averages, Volatility ");
		System.out.println("- and Percent Changes based on End of Day Prices. ");
		System.out.println("- Applies to Global Equity Instruments. ");
		System.out
				.println("- gets data for the following instrument identifiers:");
		for (int m = 0; m < cf.getInstrumentIdentifier().size(); m++) {
			System.out.println(" "
					+ cf.getInstrumentIdentifier().get(m).getIdentifierType()
					+ ": "
					+ cf.getInstrumentIdentifier().get(m).getIdentifierValue());
		}
		System.out.println("- returns data for the following output fields:");
		for (int n = 0; n < outputFields.getString().size(); n++) {
			System.out.println("  " + outputFields.getString().get(n));
		}

		System.out.println("Validating instruments...");
		InstrumentValidationResponse instrument = new InstrumentValidationResponse();
		InstrumentValidationRequest ivreq = new InstrumentValidationRequest();
		ivreq.setIdentifiers(cf);

		try {
			instrument = proxy.validateInstruments(ivreq, hch, null, hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}

		System.out.println("Performing extraction...");
		ArrayOfValidatedInstrument arrayValidated = new ArrayOfValidatedInstrument();
		arrayValidated = instrument.getInstruments();
		InstrumentExtractionRequestTechnicalIndicators request = new InstrumentExtractionRequestTechnicalIndicators();
		request.setInstruments(arrayValidated);
		request.setOutputFields(outputFields);
		ExtractionResponse response = new ExtractionResponse();

		try {
			response = proxy.extract(request, hch, null, hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}

		System.out.println("Row Count = " + response.getRowCount());
		System.out.println();
		System.out
				.println("Row Data (in CSV format - copy/paste into new file to open in a spreadsheet application) =");
		System.out.println();

		for (int i = 0; i < response.getColumns().getExtractionColumn().size(); i++) {
			System.out.print(response.getColumns().getExtractionColumn().get(i)
					.getName()
					+ ",");
		}
		System.out.println();

		for (int j = 0; j < response.getRowCount(); j++) {
			for (int i = 0; i < response.getColumns().getExtractionColumn()
					.size(); i++) {
				System.out.print(response.getColumns().getExtractionColumn()
						.get(i).getValues().getDoubleOrEmptyOrDate().get(j)
						.toString()
						+ ",");
			}
			System.out.println();
		}

		System.out.println("Extraction Messages:");
		for (int m = 0; m < response.getMessages().getString().size(); m++) {
			System.out.println(response.getMessages().getString().get(m));
		}
		System.out.println("");
	}
}
