package samples.extraction.estimates;

import javax.xml.ws.Holder;
import javax.xml.ws.soap.SOAPFaultException;

import com.reuters.datascopeselect.extractionservice.v1.*;
import com.reuters.datascopeselect.extractionservice.v1.EmptyValue;
import samples.ApiUtility;
import samples.ClientErrorException;
import samples.ServerErrorException;
import core.ISample;

public class SummaryWithLargeListSample implements ISample {
	public void Execute() throws ClientErrorException, ServerErrorException,
			Exception {
		CredentialsHeader credentials = ApiUtility.CreateCredentials();
		Holder<CredentialsHeader> hch = ApiUtility.createhch(credentials);
		Holder<OperationInfoHeader> hoih = ApiUtility.createhoih();

		com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service service = new com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service();
		com.reuters.datascopeselect.extractionservice.v1.ExtractionService proxy = service
				.getExtractionService();

		InstrumentIdentifier cf1 = new InstrumentIdentifier();
		cf1.setIdentifierType("CHR");
		cf1.setIdentifierValue("0#.N225");

		ArrayOfInstrumentIdentifier cf = new ArrayOfInstrumentIdentifier();
		cf.getInstrumentIdentifier().add(cf1);

		ArrayOfString outputFields = new ArrayOfString();
		outputFields.getString().add("RIC");
		outputFields.getString().add("Measure");
		outputFields.getString().add("Period");
		outputFields.getString().add("Period Date");
		outputFields.getString().add("Number of Estimates");
		outputFields.getString().add("High Estimate");
		outputFields.getString().add("Low Estimate");
		outputFields.getString().add("Mean Estimate");
		outputFields.getString().add("Median Estimate");

		String companyType = "Consolidated";
		String epsType = "EPS";
		String salesType = "SAL";
		String fiscalYearFrom = "FY1";
		String fiscalYearTo = "FY3";
		ArrayOfString estlist = new ArrayOfString();
		estlist.getString().add(epsType);
		estlist.getString().add(salesType);

		System.out.println("Performing an Estimate Summary extraction that:");
		System.out
				.println("- gets data for all of the equities in NIKKEI 225");
		System.out
				.println("- only gets data for " + companyType + " companies");
		System.out.println("- gets Earnings Per Share (" + epsType
				+ ") and Sales (" + salesType + ") estimates:");
		System.out
				.println("- gets estimates for the current fiscal year to the next two fiscal years ("
						+ fiscalYearFrom + " - " + fiscalYearTo + "):");
		System.out.println("- returns data for the following output fields:");
		for (int n = 0; n < outputFields.getString().size(); n++) {
			System.out.println("  " + outputFields.getString().get(n));
		}

		System.out.println("Validating instruments...");
		InstrumentValidationResponse instrument = new InstrumentValidationResponse();
		InstrumentValidationRequest ivreq = new InstrumentValidationRequest();
		ivreq.setIdentifiers(cf);
		ivreq.setPerformExpansion(true);

		try {
			instrument = proxy.validateInstruments(ivreq, hch, null, hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}
		System.out.println(Integer.toString(instrument.getInstruments()
				.getValidatedInstrument().size())
				+ " constituent RIC's returned:");
		for (int n = 0; n < instrument.getInstruments()
				.getValidatedInstrument().size(); n++) {
			System.out.println("  "
					+ instrument.getInstruments().getValidatedInstrument()
							.get(n).getRic()
					+ " "
					+ instrument.getInstruments().getValidatedInstrument()
							.get(n).getSource());
		}
		System.out
				.println("Calling the GetInstrumentExtractionTypes discoverability method");
		System.out
				.println("to dynamically determine the maximum number of instruments that can be passed to a Estimate Summary extraction...");

		ArrayOfValidatedInstrument batcharrayValidated = new ArrayOfValidatedInstrument();
		ArrayOfInstrumentExtractionTypeInfo discoResponse = new ArrayOfInstrumentExtractionTypeInfo();
		try {
			discoResponse = proxy.getInstrumentExtractionTypes(hch, null, hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}

		int maxInstruments = 0;
		for (int i = 0; i < discoResponse.getInstrumentExtractionTypeInfo()
				.size(); i++) {
			if (discoResponse.getInstrumentExtractionTypeInfo().get(i)
					.getValue().equals("EST")) {
				maxInstruments = discoResponse
						.getInstrumentExtractionTypeInfo().get(i)
						.getMaximumInstrumentsAllowed();
				break;
			}
		}
		System.out
				.println("The maximum number of instruments that can be passed is: "
						+ Integer.toString(maxInstruments));

		System.out.println("Performing extraction...");
		InstrumentExtractionRequestEstimateSummary request = new InstrumentExtractionRequestEstimateSummary();
		request.setOutputFields(outputFields);
		request.setCompanyType(companyType);
		request.setEstimateTypes(estlist);
		EstimateTimePeriodConstraintRelative TimePeriodConstraint = new EstimateTimePeriodConstraintRelative();
		TimePeriodConstraint.setFiscalYearFrom(fiscalYearFrom);
		TimePeriodConstraint.setFiscalYearTo(fiscalYearTo);
		request.setTimePeriodConstraint(TimePeriodConstraint);
		ExtractionResponse response = new ExtractionResponse();

		int currrowcount = 0;
		batcharrayValidated.getValidatedInstrument().clear();
		for (int i = 0; i < instrument.getInstruments()
				.getValidatedInstrument().size(); i++) {

			if (currrowcount == maxInstruments) {
				// Run the extraction for this batch
				request.setInstruments(batcharrayValidated);
				try {
					response = proxy.extract(request, hch, null, hoih);
				} catch (SOAPFaultException e) {
					throw ApiUtility.ConvertFaultException(e);
				}
				System.out.println("Row Count = " + response.getRowCount());
				for (int j = 0; j < response.getRowCount(); j++) {
					for (int k = 0; k < response.getColumns()
							.getExtractionColumn().size(); k++) {
						if (response.getColumns().getExtractionColumn().get(k)
								.getValues().getDoubleOrEmptyOrDate().get(j) instanceof EmptyValue) {
							System.out.print(",");
						} else {
							System.out.print(response.getColumns()
									.getExtractionColumn().get(k).getValues()
									.getDoubleOrEmptyOrDate().get(j).toString()
									+ ",");
						}
					}
					System.out.println();
				}
				batcharrayValidated.getValidatedInstrument().clear();
				currrowcount = 0;
			}
			batcharrayValidated.getValidatedInstrument()
					.add(instrument.getInstruments().getValidatedInstrument()
							.get(i));
			currrowcount++;
		}

		// Run last batch extraction
		if (batcharrayValidated.getValidatedInstrument().size() > 0) {
			request.setInstruments(batcharrayValidated);
			try {
				response = proxy.extract(request, hch, null, hoih);
			} catch (SOAPFaultException e) {
				throw ApiUtility.ConvertFaultException(e);
			}
			System.out.println("Row Count = " + response.getRowCount());
			for (int j = 0; j < response.getRowCount(); j++) {
				for (int k = 0; k < response.getColumns().getExtractionColumn()
						.size(); k++) {
					if (response.getColumns().getExtractionColumn().get(k)
							.getValues().getDoubleOrEmptyOrDate().get(j) instanceof EmptyValue) {
						System.out.print(",");
					} else {
						System.out.print(response.getColumns()
								.getExtractionColumn().get(k).getValues()
								.getDoubleOrEmptyOrDate().get(j).toString()
								+ ",");
					}
				}
				System.out.println();
			}
		}
	}
}
