package samples.extraction.legalentity;

import javax.xml.ws.Holder;
import javax.xml.ws.soap.SOAPFaultException;

import samples.ApiUtility;
import samples.ClientErrorException;
import samples.ServerErrorException;

import com.reuters.datascopeselect.extractionservice.v1.*;

import core.ISample;

public class AuditSample implements ISample {
	public void Execute() throws ClientErrorException, ServerErrorException,
			Exception {
		CredentialsHeader credentials = ApiUtility.CreateCredentials();
		Holder<CredentialsHeader> hch = ApiUtility.createhch(credentials);
		Holder<OperationInfoHeader> hoih = ApiUtility.createhoih();

		ExtractionService_Service service = new ExtractionService_Service();
		ExtractionService proxy = service.getExtractionService();

		LegalEntityIdentifier leValue1 = new LegalEntityIdentifier();
		leValue1.setIdentifierType("RCP");
		leValue1.setIdentifierValue("300012286");

		LegalEntityIdentifier leValue2 = new LegalEntityIdentifier();
		leValue2.setIdentifierType("ORG");
		leValue2.setIdentifierValue("21528");

		ArrayOfString outputFields = new ArrayOfString();
		outputFields.getString().add("RCP ID");
		outputFields.getString().add("Audit Modified Date");
		outputFields.getString().add("Audit Source");
		outputFields.getString().add("Audited Field Name");

		ArrayOfLegalEntityIdentifier leIdentifiers = new ArrayOfLegalEntityIdentifier();
		leIdentifiers.getLegalEntityIdentifier().add(leValue1);
		leIdentifiers.getLegalEntityIdentifier().add(leValue2);

		System.out.println("Performing an LegalEntity Audit extraction that:");
		System.out
				.println("- gets data for the following LegalEntity identifiers:");
		for (int m = 0; m < leIdentifiers.getLegalEntityIdentifier().size(); m++) {
			System.out.println(" "
					+ leIdentifiers.getLegalEntityIdentifier().get(m)
							.getIdentifierType()
					+ ": "
					+ leIdentifiers.getLegalEntityIdentifier().get(m)
							.getIdentifierValue());
		}
		System.out.println("- returns data for the following output fields:");
		for (int n = 0; n < outputFields.getString().size(); n++) {
			System.out.println("  " + outputFields.getString().get(n));
		}

		LegalEntityValidationResponse validateResponse = new LegalEntityValidationResponse();
		LegalEntityValidationRequest checkleIdentifiers = new LegalEntityValidationRequest();
		checkleIdentifiers.setIdentifiers(leIdentifiers);

		System.out.println("Validating Legal Entities...");
		try {
			validateResponse = proxy.validateLegalEntities(checkleIdentifiers,
					hch, null, hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}

		for (int i = 0; i < validateResponse.getLegalEntities()
				.getValidatedLegalEntity().size(); i++) {
			System.out.println(validateResponse.getLegalEntities()
					.getValidatedLegalEntity().get(i).getDescription()
					+ " : "
					+ validateResponse.getLegalEntities()
							.getValidatedLegalEntity().get(i).getOrgId());
		}

		System.out.println("Performing extraction...");

		LegalEntityExtractionRequestAudit request = new LegalEntityExtractionRequestAudit();
		request.setLegalEntities(validateResponse.getLegalEntities());
		request.setOutputFields(outputFields);
		ExtractionResponse response = new ExtractionResponse();

		try {
			response = proxy.extract(request, hch, null, hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}

		System.out.println("Row Count = " + response.getRowCount());
		System.out
				.println("Row Data (in CSV format - copy/paste into new file to open in a spreadsheet application) = ");
		for (int j = 0; j < response.getColumns().getExtractionColumn().size(); j++) {
			System.out.print('"'
					+ response.getColumns().getExtractionColumn().get(j)
							.getName().toString() + '"' + ",");
		}
		System.out.println("");

		for (int k = 0; k < response.getRowCount(); k++) {
			for (int l = 0; l < response.getColumns().getExtractionColumn()
					.size(); l++) {
				System.out.print('"'
						+ response.getColumns().getExtractionColumn().get(l)
								.getValues().getDoubleOrEmptyOrDate().get(k)
								.toString() + '"' + ",");
			}
			System.out.println("");
		}

		System.out.println("Extraction Messages:");
		for (int m = 0; m < response.getMessages().getString().size(); m++) {
			System.out.println(response.getMessages().getString().get(m));
		}
		System.out.println("");
	}
}
