package samples.extraction.legalentity;

import javax.xml.ws.Holder;
import javax.xml.ws.soap.SOAPFaultException;

import samples.ApiUtility;
import samples.ClientErrorException;
import samples.ServerErrorException;

import com.reuters.datascopeselect.extractionservice.v1.*;

import core.ISample;

public class DetailSample implements ISample {
	public void Execute() throws ClientErrorException, ServerErrorException,
			Exception {
		CredentialsHeader credentials = ApiUtility.CreateCredentials();
		Holder<CredentialsHeader> hch = ApiUtility.createhch(credentials);
		Holder<OperationInfoHeader> hoih = ApiUtility.createhoih();
		
		com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service service = new com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service();
		com.reuters.datascopeselect.extractionservice.v1.ExtractionService proxy = service
				.getExtractionService();

		LegalEntityIdentifier cpvalue1 = new LegalEntityIdentifier();
		cpvalue1.setIdentifierType("RCP");
		cpvalue1.setIdentifierValue("300012286");

		LegalEntityIdentifier cpvalue2 = new LegalEntityIdentifier();
		cpvalue2.setIdentifierType("ORG");
		cpvalue2.setIdentifierValue("21528");
		
		int daysLimit = 365;
		ArrayOfString outputFields = new ArrayOfString();
		outputFields.getString().add("RCP ID");
		outputFields.getString().add("Legal Entity Type");
		outputFields.getString().add("Official Name");
		outputFields.getString().add("Domicile");

		ArrayOfLegalEntityIdentifier cpidentifiers = new ArrayOfLegalEntityIdentifier();
		cpidentifiers.getLegalEntityIdentifier().add(cpvalue1);
		cpidentifiers.getLegalEntityIdentifier().add(cpvalue2);

		System.out.println("Performing an LegalEntity Detail extraction that:");
		System.out
				.println("- gets data for the following LegalEntity identifiers:");
		for (int m = 0; m < cpidentifiers.getLegalEntityIdentifier().size(); m++) {
			System.out.println(" "
					+ cpidentifiers.getLegalEntityIdentifier().get(m)
							.getIdentifierType()
					+ ": "
					+ cpidentifiers.getLegalEntityIdentifier().get(m)
							.getIdentifierValue());
		}
		System.out.println("- that have occurred within the last " + daysLimit + " days.");
		System.out.println("- returns data for the following output fields:");
		for (int n = 0; n < outputFields.getString().size(); n++) {
			System.out.println("  " + outputFields.getString().get(n));
		}

		LegalEntityValidationResponse validateresponse = new LegalEntityValidationResponse();
		LegalEntityValidationRequest checkcpidentifiers = new LegalEntityValidationRequest();
		checkcpidentifiers.setIdentifiers(cpidentifiers);

		System.out.println("Validating LegalEntities...");
		try {
			validateresponse = proxy.validateLegalEntities(checkcpidentifiers,
					hch, null, hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}

		for (int i = 0; i < validateresponse.getLegalEntities()
				.getValidatedLegalEntity().size(); i++) {
			System.out.println(validateresponse.getLegalEntities()
					.getValidatedLegalEntity().get(i).getDescription()
					+ " : "
					+ validateresponse.getLegalEntities()
							.getValidatedLegalEntity().get(i).getOrgId());
		}

		System.out.println("Performing extraction...");
		LegalEntityExtractionRequestDetail request = new LegalEntityExtractionRequestDetail();
		request.setLegalEntities(validateresponse.getLegalEntities());
		request.setOutputFields(outputFields);
		LegalEntityDetailConstraintDaysAgoDelta legalEntityDaysAgo = new LegalEntityDetailConstraintDaysAgoDelta();
		legalEntityDaysAgo.setDaysAgo(daysLimit);
		request.setConstraint(legalEntityDaysAgo);		
		
		ExtractionResponse response = new ExtractionResponse();

		try {
			response = proxy.extract(request, hch, null, hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}

		System.out.println("Row Count = " + response.getRowCount());
		System.out
				.println("Row Data (in CSV format - copy/paste into new file to open in a spreadsheet application) = ");
		for (int j = 0; j < response.getColumns().getExtractionColumn().size(); j++) {
			System.out.print('"'
					+ response.getColumns().getExtractionColumn().get(j)
							.getName().toString() + '"' + ",");
		}
		System.out.println("");

		for (int k = 0; k < response.getRowCount(); k++) {
			for (int l = 0; l < response.getColumns().getExtractionColumn()
					.size(); l++) {
				System.out.print('"'
						+ response.getColumns().getExtractionColumn().get(l)
								.getValues().getDoubleOrEmptyOrDate().get(k)
								.toString() + '"' + ",");
			}
			System.out.println("");
		}

		System.out.println("Extraction Messages:");
		for (int m = 0; m < response.getMessages().getString().size(); m++) {
			System.out.println(response.getMessages().getString().get(m));
		}
		System.out.println("");
	}

}
