package samples.extraction.legalentity;

import javax.xml.ws.Holder;
import javax.xml.ws.soap.SOAPFaultException;

import com.reuters.datascopeselect.extractionservice.v1.*;

import samples.ApiUtility;
import samples.ClientErrorException;
import samples.ServerErrorException;
import core.ISample;

public class HierarchySample implements ISample {
	public void Execute() throws ClientErrorException, ServerErrorException,
			Exception {
		CredentialsHeader credentials = ApiUtility.CreateCredentials();
		Holder<CredentialsHeader> hch = ApiUtility.createhch(credentials);
		Holder<OperationInfoHeader> hoih = ApiUtility.createhoih();

		com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service service = new com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service();
		com.reuters.datascopeselect.extractionservice.v1.ExtractionService proxy = service
				.getExtractionService();

		LegalEntityIdentifier cpvalue1 = new LegalEntityIdentifier();
		cpvalue1.setIdentifierType("ORG");
		cpvalue1.setIdentifierValue("21528");

		ArrayOfLegalEntityIdentifier cpidentifiers = new ArrayOfLegalEntityIdentifier();
		cpidentifiers.getLegalEntityIdentifier().add(cpvalue1);

		ArrayOfString outputFields = new ArrayOfString();
		outputFields.getString().add("RCP ID");
		outputFields.getString().add("Official Name");
		outputFields.getString().add("Node/Level");
		outputFields.getString().add("Relationship Type");
		outputFields.getString().add("Related Entity ID");
		outputFields.getString().add("Related Official Name");
		outputFields.getString().add("Entity Is Managed");
		outputFields.getString().add("Entity Is Verified");

		System.out
				.println("Performing an LegalEntity Hierarchy extraction that:");
		System.out
				.println("- gets data for the following LegalEntity identifiers:");
		for (int m = 0; m < cpidentifiers.getLegalEntityIdentifier().size(); m++) {
			System.out.println(" "
					+ cpidentifiers.getLegalEntityIdentifier().get(m)
							.getIdentifierType()
					+ ": "
					+ cpidentifiers.getLegalEntityIdentifier().get(m)
							.getIdentifierValue());
		}
		System.out.println("- returns data for the following output fields:");
		for (int n = 0; n < outputFields.getString().size(); n++) {
			System.out.println("  " + outputFields.getString().get(n));
		}

		LegalEntityValidationResponse validateresponse = new LegalEntityValidationResponse();
		LegalEntityValidationRequest checkcpidentifiers = new LegalEntityValidationRequest();
		checkcpidentifiers.setIdentifiers(cpidentifiers);

		System.out.println("Validating LegalEntities...");
		try {
			validateresponse = proxy.validateLegalEntities(checkcpidentifiers, hch, null, hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}

		System.out.println("Performing extraction...");

		LegalEntityExtractionRequestHierarchy request = new LegalEntityExtractionRequestHierarchy();
		request.setLegalEntities(validateresponse.getLegalEntities());
		request.setOutputFields(outputFields);
		ExtractionResponse response = new ExtractionResponse();

		try {
			response = proxy.extract(request, hch, null, hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}

		System.out.println("Row Count = " + response.getRowCount());
		System.out
				.println("Row Data (in CSV format - copy/paste into new file to open in a spreadsheet application) = ");
		for (int j = 0; j < response.getColumns().getExtractionColumn().size(); j++) {
			System.out.print('"'
					+ response.getColumns().getExtractionColumn().get(j)
							.getName().toString() + '"' + ",");
		}

		System.out.println("");
		for (int j = 0; j < response.getRowCount(); j++) {
			for (int i = 0; i < response.getColumns().getExtractionColumn()
					.size(); i++) {
				if (response.getColumns().getExtractionColumn().get(i)
						.getValues().getDoubleOrEmptyOrDate().get(j) instanceof EmptyValue) {
					System.out.print(",");
				} else {
					System.out.print(response.getColumns()
							.getExtractionColumn().get(i).getValues()
							.getDoubleOrEmptyOrDate().get(j).toString()
							+ ",");
				}
			}
			System.out.println();
		}

		System.out.println("");

		System.out.println("Extraction Messages:");
		for (int m = 0; m < response.getMessages().getString().size(); m++) {
			System.out.println(response.getMessages().getString().get(m));
		}
		System.out.println("");
	}


	public static int tryParse(Object value) {
		try {
			return Integer.parseInt(value.toString());
		} catch (NumberFormatException ex) {
			return 0;
		}
	}

	public static int fieldsIndexesByName(String fieldname,
			ArrayOfString outputFields) {
		int n = 0;
		for (n = 0; n < outputFields.getString().size(); n++) {
			if (fieldname.equals(outputFields.getString().get(n))) {
				break;
			}
		}
		return n;
	}
}