package samples.extraction.other;

import javax.xml.ws.Holder;
import javax.xml.ws.soap.SOAPFaultException;

import com.reuters.datascopeselect.extractionservice.v1.*;

import samples.ApiUtility;
import samples.ClientErrorException;
import samples.ExtractionUtility;
import samples.ServerErrorException;
import core.ISample;

public class CompositeSample implements ISample {

	public void Execute() throws ClientErrorException, ServerErrorException,
			Exception {
		CredentialsHeader credentials = ApiUtility.CreateCredentials();
		Holder<CredentialsHeader> hch = ApiUtility.createhch(credentials);
		Holder<OperationInfoHeader> hoih = ApiUtility.createhoih();

		com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service service = new com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service();
		com.reuters.datascopeselect.extractionservice.v1.ExtractionService proxy = service
				.getExtractionService();

		InstrumentIdentifier inst1 = new InstrumentIdentifier();
		inst1.setIdentifierType("RIC");
		inst1.setIdentifierValue("KO.N");

		InstrumentIdentifier inst2 = new InstrumentIdentifier();
		inst2.setIdentifierType("RIC");
		inst2.setIdentifierValue("IBM.N");

		ArrayOfInstrumentIdentifier instInput = new ArrayOfInstrumentIdentifier();
		instInput.getInstrumentIdentifier().add(inst1);
		instInput.getInstrumentIdentifier().add(inst2);

		ArrayOfString outputFields = new ArrayOfString();
		outputFields.getString().add("RIC");
		outputFields.getString().add("Universal Ask Price");
		outputFields.getString().add("Universal Bid Price");
		outputFields.getString().add("Universal Close Price");

		System.out.println("Performing a Composite extraction that:");
		System.out.println("- gets data for the following instrument identifiers:");
		for (int m = 0; m < instInput.getInstrumentIdentifier().size(); m++) {
			System.out.println(" "
					+ instInput.getInstrumentIdentifier().get(m).getIdentifierType()
					+ ": "
					+ instInput.getInstrumentIdentifier().get(m).getIdentifierValue());
		}
		System.out.println("- returns data for the following output fields:");
		for (int n = 0; n < outputFields.getString().size(); n++) {
			System.out.println("  " + outputFields.getString().get(n));
		}
		
		System.out.println("Validating instruments...");
		InstrumentValidationResponse instrument = new InstrumentValidationResponse();
		InstrumentValidationRequest ivreq = new InstrumentValidationRequest();
		ivreq.setIdentifiers(instInput);

		try {
			instrument = proxy.validateInstruments(ivreq, hch, null, hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}

		System.out.println("Performing extraction...");
		ArrayOfValidatedInstrument arrayValidated = new ArrayOfValidatedInstrument();
		arrayValidated = instrument.getInstruments();
		InstrumentExtractionRequestComposite request = new InstrumentExtractionRequestComposite();
		request.setInstruments(arrayValidated);
		request.setOutputFields(outputFields);

		ExtractionResponse response = new ExtractionResponse();
		try {
			response = proxy.extract(request, hch, null, hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}

		System.out.println("Row Count = " + response.getRowCount());
		System.out.println();
		System.out
				.println("Row Data (in CSV format - copy/paste into new file to open in a spreadsheet application) =");
		System.out.println();

		for (int i = 0; i < response.getColumns().getExtractionColumn().size(); i++) {
			System.out.print(response.getColumns().getExtractionColumn().get(i)
					.getName()
					+ ",");
		}
		System.out.println();

		for (int j = 0; j < response.getRowCount(); j++) {
			for (int i = 0; i < response.getColumns().getExtractionColumn()
					.size(); i++) {
				System.out.print(response.getColumns().getExtractionColumn()
						.get(i).getValues().getDoubleOrEmptyOrDate().get(j)
						.toString()
						+ ",");
			}
			System.out.println();
		}

		System.out.println("Extraction Messages:");
		for (int m = 0; m < response.getMessages().getString().size(); m++) {
			System.out.println(response.getMessages().getString().get(m));
		}
		System.out.println("");

	}	
}