package samples.extraction.other;

import javax.xml.ws.Holder;
import javax.xml.ws.soap.SOAPFaultException;

import com.reuters.datascopeselect.extractionservice.v1.*;

import samples.ApiUtility;
import samples.ClientErrorException;
import samples.ExtractionUtility;
import samples.ServerErrorException;
import core.ISample;

public class FundAllocationSample implements ISample {
	public void Execute() throws ClientErrorException, ServerErrorException,
			Exception {
		CredentialsHeader credentials = ApiUtility.CreateCredentials();
		Holder<CredentialsHeader> hch = ApiUtility.createhch(credentials);
		Holder<OperationInfoHeader> hoih = ApiUtility.createhoih();
		com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service service = new com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service();
		com.reuters.datascopeselect.extractionservice.v1.ExtractionService proxy = service
				.getExtractionService();

		ArrayOfString outputFields = new ArrayOfString();
		outputFields.getString().add("Allocation Date");
		outputFields.getString().add("Allocation Item");
		outputFields.getString().add("Allocation Percentage");
		outputFields.getString().add("Allocation Rank");
		outputFields.getString().add("Allocation Type");
		outputFields.getString().add("Asset Type");
		outputFields.getString().add("Asset Type Description");
		outputFields.getString().add("CUSIP");
		outputFields.getString().add("FundSERVAC");
		outputFields.getString().add("FundSERVBE");
		outputFields.getString().add("FundSERVDM");
		outputFields.getString().add("FundSERVDO");
		outputFields.getString().add("FundSERVFE");
		outputFields.getString().add("FundSERVIS");
		outputFields.getString().add("FundSERVLL");
		outputFields.getString().add("FundSERVNL");
		outputFields.getString().add("FundSERVVS");
		outputFields.getString().add("Identifier Segment Source");
		outputFields.getString().add("Identifier Segment Source Description");
		outputFields.getString().add("Instrument ID");
		outputFields.getString().add("Instrument ID Type");
		outputFields.getString().add("ISIN");
		outputFields.getString().add("Lipper ID");
		outputFields.getString().add("PILC");
		outputFields.getString().add("RIC");
		outputFields.getString().add("Security Description");
		outputFields.getString().add("SEDOL");
		outputFields.getString().add("Usage Instrument SubType");
		outputFields.getString().add("Usage Instrument Type");
		outputFields.getString().add("User Defined Identifier");
		outputFields.getString().add("User Defined Identifier2");
		outputFields.getString().add("User Defined Identifier3");
		outputFields.getString().add("Valoren");
		outputFields.getString().add("Wertpapier");


		System.out.println("Performing a Fund Allocation extraction that:");
		System.out.println("- Retrieves one or more fund allocation schemes, including asset, country, currency, ");
		System.out.println("- sector and top ten holding, for Lipper Fund instruments.");
		System.out.println("- returns data for the following output fields:");
		for (int n = 0; n < outputFields.getString().size(); n++) {
			System.out.println("  " + outputFields.getString().get(n));
		}

		String lipperId = "60000012";
		
		System.out.println("Searching for Funds that:");
		System.out.println("- have a LipperID of {" + lipperId + "}");

		InstrumentSearchRequestAll searchRequest = new InstrumentSearchRequestAll(); // TODO: This is temporary. The correct search functionality will be implemented in 9.2
		searchRequest.setIdentifierType("LIP");
		searchRequest.setIdentifierSearchValue(lipperId);
		searchRequest.setPreferredIdentifierType("LIP");

		InstrumentSearchResponse searchresponse = new InstrumentSearchResponse();
		try {
			searchresponse = proxy.searchInstruments(searchRequest, hch, null,
					hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}
		System.out.println(Integer.toString(searchresponse.getInstruments()
				.getValidatedSearchInstrument().size())
				+ " instruments found:");
		for (int n = 0; n < searchresponse.getInstruments()
				.getValidatedSearchInstrument().size(); n++) {
			System.out.println("  "
					+ searchresponse.getInstruments()
							.getValidatedSearchInstrument().get(n)
							.getIdentifierType()
					+ ": "
					+ searchresponse.getInstruments()
							.getValidatedSearchInstrument().get(n)
							.getIdentifierValue()
					+ " "
					+ searchresponse.getInstruments()
							.getValidatedSearchInstrument().get(n).getSource());
		}

		System.out.println("Performing extraction...");
		InstrumentExtractionRequestFundAllocation request = new InstrumentExtractionRequestFundAllocation();
		FundAllocationFieldConstraint constraints = new FundAllocationFieldConstraint();
		ArrayOfValidatedInstrument arrayValidated = ExtractionUtility
				.ChangeDataType(searchresponse.getInstruments());

		request.setOutputFields(outputFields);
		request.setInstruments(arrayValidated);
		
		ArrayOfString allocationTypes = new ArrayOfString();
		allocationTypes.getString().add("ASSET");
		allocationTypes.getString().add("CNTRY");
		allocationTypes.getString().add("CURR");
		allocationTypes.getString().add("INDUS");
		allocationTypes.getString().add("HLD10");
		
		constraints.setAllocationTypes(allocationTypes);
		request.setFieldConstraint(constraints);

		ExtractionResponse response = new ExtractionResponse();
		try {
			response = proxy.extract(request, hch, null, hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}

		System.out.println("Row Count = " + response.getRowCount());
		System.out.println();
		System.out
				.println("Row Data (in CSV format - copy/paste into new file to open in a spreadsheet application) =");
		System.out.println();

		for (int i = 0; i < response.getColumns().getExtractionColumn().size(); i++) {
			System.out.print(response.getColumns().getExtractionColumn().get(i)
					.getName()
					+ ",");
		}
		System.out.println();

		for (int j = 0; j < response.getRowCount(); j++) {
			for (int i = 0; i < response.getColumns().getExtractionColumn()
					.size(); i++) {
				if (response.getColumns().getExtractionColumn().get(i)
						.getValues().getDoubleOrEmptyOrDate().get(j) instanceof EmptyValue) {
					System.out.print(",");
				} else {
					System.out.print(response.getColumns()
							.getExtractionColumn().get(i).getValues()
							.getDoubleOrEmptyOrDate().get(j).toString()
							+ ",");
				}
			}
			System.out.println();
		}
		ExtractionUtility.EnsureOutputFields(response);

		System.out.println("Extraction Messages:");
		for (int m = 0; m < response.getMessages().getString().size(); m++) {
			System.out.println(response.getMessages().getString().get(m));
		}
		System.out.println("");
	}
}
