package samples.extraction.other;

import javax.xml.ws.Holder;
import javax.xml.ws.soap.SOAPFaultException;

import com.reuters.datascopeselect.extractionservice.v1.*;

import samples.ApiUtility;
import samples.ClientErrorException;
import samples.ExtractionUtility;
import samples.ServerErrorException;
import core.ISample;

public class RatingsSample implements ISample{

	public void Execute() throws ClientErrorException, ServerErrorException,
			Exception {
		CredentialsHeader credentials = ApiUtility.CreateCredentials();
		Holder<CredentialsHeader> hch = ApiUtility.createhch(credentials);
		Holder<OperationInfoHeader> hoih = ApiUtility.createhoih();
		com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service service = new com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service();
		com.reuters.datascopeselect.extractionservice.v1.ExtractionService proxy = service
				.getExtractionService();
		
		ArrayOfString outputFields = new ArrayOfString();
		outputFields.getString().add("Asset ID");
		outputFields.getString().add("Bond Type Description");
		outputFields.getString().add("CUSIP");
		outputFields.getString().add("Issue Rating Code");
		outputFields.getString().add("Rating Type");
		
		ArrayOfString ratingSources = new ArrayOfString();
		ratingSources.getString().add("S&P");
		
		RatingsQueryConstraintLastUpdated ratingsQueryConstraintLastUpdated = new RatingsQueryConstraintLastUpdated();
		ratingsQueryConstraintLastUpdated.setIncludeInstrumentsWithNoRatings(true);
		
		RatingsFieldConstraint fieldConstraint = new RatingsFieldConstraint();
		fieldConstraint.setIncludeIssueRatingLevel(true);
		fieldConstraint.setSources(ratingSources);
		
		
		
		System.out.println("Performing a Ratings extraction that:");
		System.out.println("- gets its instruments from a Gov/Corp search");
		System.out.println("- that have an issue level rating");
		System.out.println("- from the following data sources");
		for (int n = 0; n < ratingSources.getString().size(); n++) {
			System.out.println("  " + ratingSources.getString().get(n));
		}
		System.out.println("- returns data for the following output fields");
		for (int n = 0; n < outputFields.getString().size(); n++) {
			System.out.println("  " + outputFields.getString().get(n));
		}
		
		String spType = "AAA";
		String country = "US";
		ArrayOfString splist = new ArrayOfString();
		splist.getString().add(spType);
		
		System.out.println("Searching for Gov/Corp Instruments that:");
		System.out.println("- are corporate bonds");
		System.out.println("- have a S&P rating of {" + spType + "}");
		System.out.println("- are from the United States (" + country + ")");
				
		InstrumentSearchRequestGovCorp searchRequest = new InstrumentSearchRequestGovCorp();
		searchRequest.setIdentifierType("CSP");
		searchRequest.setIdentifierSearchValue("*");
		searchRequest.setPreferredIdentifierType("CSP");
		searchRequest.setIsCorporateGroup(true);
		searchRequest.setStandardPoorsTypes(splist);
		searchRequest.setCountryType(country);
		searchRequest.setIsCallable(true);
		searchRequest.setIsPutable(true);
		
		InstrumentSearchResponse searchresponse = new InstrumentSearchResponse();
		try {
			searchresponse = proxy.searchInstruments(searchRequest, hch, null,
					hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}
		System.out.println(Integer.toString(searchresponse.getInstruments()
				.getValidatedSearchInstrument().size())
				+ " instruments found:");
		for (int n = 0; n < searchresponse.getInstruments()
				.getValidatedSearchInstrument().size(); n++) {
			System.out.println("  "
					+ searchresponse.getInstruments()
							.getValidatedSearchInstrument().get(n)
							.getIdentifierType()
					+ ": "
					+ searchresponse.getInstruments()
							.getValidatedSearchInstrument().get(n)
							.getIdentifierValue()
					+ " "
					+ searchresponse.getInstruments()
							.getValidatedSearchInstrument().get(n).getSource());
		}
		
		System.out.println("Performing extraction...");
		InstrumentExtractionRequestRatings request = new InstrumentExtractionRequestRatings();
		ArrayOfValidatedInstrument arrayValidated = ExtractionUtility.ChangeDataType(searchresponse.getInstruments());
		
		request.setOutputFields(outputFields);
		request.setInstruments(arrayValidated);
		request.setFieldConstraint(fieldConstraint);
		request.setQueryConstraint(ratingsQueryConstraintLastUpdated);
		
		ExtractionResponse response = new ExtractionResponse();
		try {
			response = proxy.extract(request, hch, null, hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}
		
		System.out.println("Row Count = " + response.getRowCount());
		System.out.println();
		System.out
				.println("Row Data (in CSV format - copy/paste into new file to open in a spreadsheet application) =");
		System.out.println();
		
		for (int i = 0; i < response.getColumns().getExtractionColumn().size(); i++) {
			System.out.print(response.getColumns().getExtractionColumn().get(i)
					.getName()
					+ ",");
		}
		System.out.println();
		
		for (int j = 0; j < response.getRowCount(); j++) {
			for (int i = 0; i < response.getColumns().getExtractionColumn()
					.size(); i++) {
				if (response.getColumns().getExtractionColumn().get(i)
						.getValues().getDoubleOrEmptyOrDate().get(j) instanceof EmptyValue) {
					System.out.print(",");
				} else {
					System.out.print(response.getColumns()
							.getExtractionColumn().get(i).getValues()
							.getDoubleOrEmptyOrDate().get(j).toString()
							+ ",");
				}
			}
			System.out.println();
		}
		ExtractionUtility.EnsureOutputFields(response);
		
		System.out.println("Extraction Messages:");
		for (int m = 0; m < response.getMessages().getString().size(); m++) {
			System.out.println(response.getMessages().getString().get(m));
		}
		System.out.println("");
		}

}
