package samples.extraction.other;

import javax.xml.ws.Holder;
import javax.xml.ws.soap.SOAPFaultException;

import com.reuters.datascopeselect.extractionservice.v1.*;

import samples.ApiUtility;
import samples.ClientErrorException;
import samples.ExtractionUtility;
import samples.ServerErrorException;
import core.ISample;

public class TermsAndConditionsSample implements ISample {
	public void Execute() throws ClientErrorException, ServerErrorException,
			Exception {
		CredentialsHeader credentials = ApiUtility.CreateCredentials();
		Holder<CredentialsHeader> hch = ApiUtility.createhch(credentials);
		Holder<OperationInfoHeader> hoih = ApiUtility.createhoih();

		com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service service = new com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service();
		com.reuters.datascopeselect.extractionservice.v1.ExtractionService proxy = service
				.getExtractionService();

		ArrayOfString outputFields = new ArrayOfString();
		outputFields.getString().add("CUSIP");
		outputFields.getString().add("Coupon Rate");
		outputFields.getString().add("Coupon Frequency");
		outputFields.getString().add("Maturity Date");
		outputFields.getString().add("Moodys Rating");

		System.out
				.println("Performing a Terms and Conditions extraction that:");
		System.out.println("- gets its instruments from a Gov/Corp search");
		System.out.println("- returns data for the following output fields:");
		for (int n = 0; n < outputFields.getString().size(); n++) {
			System.out.println("  " + outputFields.getString().get(n));
		}

		String spType = "AAA";
		String country = "US";
		ArrayOfString splist = new ArrayOfString();
		splist.getString().add(spType);

		System.out.println("Searching for Gov/Corp Instruments that:");
		System.out.println("- are corporate bonds");
		System.out.println("- have a S&P rating of (" + spType + ")");
		System.out.println("- are from the United States (" + country + ")");
		System.out.println("- are callable");
		System.out.println("- are putable");

		InstrumentSearchRequestGovCorp searchRequest = new InstrumentSearchRequestGovCorp();
		searchRequest.setIdentifierType("CSP");
		searchRequest.setIdentifierSearchValue("*");
		searchRequest.setPreferredIdentifierType("CSP");
		searchRequest.setIsCorporateGroup(true);
		searchRequest.setStandardPoorsTypes(splist);
		searchRequest.setCountryType(country);
		searchRequest.setIsCallable(true);
		searchRequest.setIsPutable(true);

		InstrumentSearchResponse searchresponse = new InstrumentSearchResponse();
		try {
			searchresponse = proxy.searchInstruments(searchRequest, hch, null,
					hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}
		System.out.println(Integer.toString(searchresponse.getInstruments()
				.getValidatedSearchInstrument().size())
				+ " instruments found:");
		for (int n = 0; n < searchresponse.getInstruments()
				.getValidatedSearchInstrument().size(); n++) {
			System.out.println("  "
					+ searchresponse.getInstruments()
							.getValidatedSearchInstrument().get(n)
							.getIdentifierType()
					+ ": "
					+ searchresponse.getInstruments()
							.getValidatedSearchInstrument().get(n)
							.getIdentifierValue()
					+ " "
					+ searchresponse.getInstruments()
							.getValidatedSearchInstrument().get(n).getSource());
		}

		System.out.println("Performing extraction...");
		InstrumentExtractionRequestTermsAndConditions request = new InstrumentExtractionRequestTermsAndConditions();
		ArrayOfValidatedInstrument arrayValidated = ExtractionUtility
				.ChangeDataType(searchresponse.getInstruments());

		request.setOutputFields(outputFields);
		request.setInstruments(arrayValidated);

		ExtractionResponse response = new ExtractionResponse();
		try {
			response = proxy.extract(request, hch, null, hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}

		System.out.println("Row Count = " + response.getRowCount());
		System.out.println();
		System.out
				.println("Row Data (in CSV format - copy/paste into new file to open in a spreadsheet application) =");
		System.out.println();

		for (int i = 0; i < response.getColumns().getExtractionColumn().size(); i++) {
			System.out.print(response.getColumns().getExtractionColumn().get(i)
					.getName()
					+ ",");
		}
		System.out.println();

		for (int j = 0; j < response.getRowCount(); j++) {
			for (int i = 0; i < response.getColumns().getExtractionColumn()
					.size(); i++) {
				if (response.getColumns().getExtractionColumn().get(i)
						.getValues().getDoubleOrEmptyOrDate().get(j) instanceof EmptyValue) {
					System.out.print(",");
				} else {
					System.out.print(response.getColumns()
							.getExtractionColumn().get(i).getValues()
							.getDoubleOrEmptyOrDate().get(j).toString()
							+ ",");
				}
			}
			System.out.println();
		}
		ExtractionUtility.EnsureOutputFields(response);

		System.out.println("Extraction Messages:");
		for (int m = 0; m < response.getMessages().getString().size(); m++) {
			System.out.println(response.getMessages().getString().get(m));
		}
		System.out.println("");
	}
}
