package samples.extraction.pricing;

import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.ws.Holder;
import javax.xml.ws.soap.SOAPFaultException;

import com.reuters.datascopeselect.extractionservice.v1.*;

import samples.ApiUtility;
import samples.ClientErrorException;
import samples.JavaUtility;
import samples.ServerErrorException;
import core.ISample;

public class TimeSeriesWithLargeListSample implements ISample {
	public void Execute() throws ClientErrorException, ServerErrorException,
			Exception {
		CredentialsHeader credentials = ApiUtility.CreateCredentials();
		Holder<CredentialsHeader> hch = ApiUtility.createhch(credentials);
		Holder<OperationInfoHeader> hoih = ApiUtility.createhoih();

		com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service service = new com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service();
		com.reuters.datascopeselect.extractionservice.v1.ExtractionService proxy = service
				.getExtractionService();

		InstrumentIdentifier cf1 = new InstrumentIdentifier();
		cf1.setIdentifierType("RIC");
		cf1.setIdentifierValue("GOOG.O");
		InstrumentIdentifier cf2 = new InstrumentIdentifier();
		cf2.setIdentifierType("RIC");
		cf2.setIdentifierValue("IBM.N");
		InstrumentIdentifier cf3 = new InstrumentIdentifier();
		cf3.setIdentifierType("RIC");
		cf3.setIdentifierValue("KO.N");
		InstrumentIdentifier cf4 = new InstrumentIdentifier();
		cf4.setIdentifierType("RIC");
		cf4.setIdentifierValue("AAPL.O");
		InstrumentIdentifier cf5 = new InstrumentIdentifier();
		cf5.setIdentifierType("RIC");
		cf5.setIdentifierValue("TRI.N");

		ArrayOfInstrumentIdentifier cf = new ArrayOfInstrumentIdentifier();
		cf.getInstrumentIdentifier().add(cf1);
		cf.getInstrumentIdentifier().add(cf2);
		cf.getInstrumentIdentifier().add(cf3);
		cf.getInstrumentIdentifier().add(cf4);
		cf.getInstrumentIdentifier().add(cf5);

		ArrayOfString outputFields = new ArrayOfString();
		outputFields.getString().add("RIC");
		outputFields.getString().add("Trade Date");
		outputFields.getString().add("Security Description");
		outputFields.getString().add("Ask Price");
		outputFields.getString().add("Bid Price");
		outputFields.getString().add("Ask Yield");
		outputFields.getString().add("Bid Yield");
		outputFields.getString().add("Ask Discount");
		outputFields.getString().add("Bid Discount");
		outputFields.getString().add("Close Price");
		outputFields.getString().add("High Price");
		outputFields.getString().add("Low Price");

		XMLGregorianCalendar startDate = JavaUtility
				.asXMLGregorianCalendar(JavaUtility.dateadd("DAY", -7));
		XMLGregorianCalendar endDate = JavaUtility
				.asXMLGregorianCalendar(JavaUtility.dateadd("DAY", 0));

		System.out.println("Performing an TimeSeries Pricing extraction that:");
		System.out.println("- gets data for multiple US Equities");
		System.out.println("- gets history for the last week ("
				+ JavaUtility.printCalendar(startDate.toGregorianCalendar())
				+ " - "
				+ JavaUtility.printCalendar(endDate.toGregorianCalendar())
				+ ")");
		System.out.println("- returns data for the following output fields:");
		for (int n = 0; n < outputFields.getString().size(); n++) {
			System.out.println("  " + outputFields.getString().get(n));
		}

		System.out.println("Validating instruments...");
		InstrumentValidationResponse instrument = new InstrumentValidationResponse();
		InstrumentValidationRequest ivreq = new InstrumentValidationRequest();
		ivreq.setIdentifiers(cf);
		ivreq.setPerformExpansion(true);

		try {
			instrument = proxy.validateInstruments(ivreq, hch, null, hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}
		System.out.println(Integer.toString(instrument.getInstruments()
				.getValidatedInstrument().size())
				+ " RIC's returned:");
		for (int n = 0; n < instrument.getInstruments()
				.getValidatedInstrument().size(); n++) {
			System.out.println("  "
					+ instrument.getInstruments().getValidatedInstrument()
							.get(n).getRic()
					+ " "
					+ instrument.getInstruments().getValidatedInstrument()
							.get(n).getSource());
		}
		System.out
				.println("Calling the GetInstrumentExtractionTypes discoverability method");
		System.out
				.println("to dynamically determine the maximum number of instruments that can be passed to a TimeSeries Pricing extraction...");
		ArrayOfValidatedInstrument batcharrayValidated = new ArrayOfValidatedInstrument();
		ArrayOfInstrumentExtractionTypeInfo discoResponse = new ArrayOfInstrumentExtractionTypeInfo();

		try {
			discoResponse = proxy.getInstrumentExtractionTypes(hch, null, hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}

		int maxInstruments = 0;
		for (int i = 0; i < discoResponse.getInstrumentExtractionTypeInfo()
				.size(); i++) {
			if (discoResponse.getInstrumentExtractionTypeInfo().get(i)
					.getValue().equals("TPX")) {
				maxInstruments = discoResponse
						.getInstrumentExtractionTypeInfo().get(i)
						.getMaximumInstrumentsAllowed();
				break;
			}
		}
		System.out
				.println("The maximum number of instruments that can be passed is: "
						+ Integer.toString(maxInstruments));

		InstrumentExtractionRequestTimeSeriesPricing request = new InstrumentExtractionRequestTimeSeriesPricing();
		request.setOutputFields(outputFields);
		request.setStartDate(startDate);
		request.setEndDate(endDate);

		System.out.println("Performing extraction...");
		ExtractionResponse response = new ExtractionResponse();

		int currrowcount = 0;
		batcharrayValidated.getValidatedInstrument().clear();
		for (int i = 0; i < instrument.getInstruments()
				.getValidatedInstrument().size(); i++) {

			if (currrowcount == maxInstruments) {
				// Run the extraction for this batch
				request.setInstruments(batcharrayValidated);
				try {
					response = proxy.extract(request, hch, null, hoih);
				} catch (SOAPFaultException e) {
					throw ApiUtility.ConvertFaultException(e);
				}
				System.out.println("Row Count = " + response.getRowCount());
				for (int j = 0; j < response.getRowCount(); j++) {
					for (int k = 0; k < response.getColumns()
							.getExtractionColumn().size(); k++) {
						if (response.getColumns().getExtractionColumn().get(k)
								.getValues().getDoubleOrEmptyOrDate().get(j) instanceof EmptyValue) {
							System.out.print(",");
						} else {
							System.out.print(response.getColumns()
									.getExtractionColumn().get(k).getValues()
									.getDoubleOrEmptyOrDate().get(j).toString()
									+ ",");
						}
					}
					System.out.println();
				}
				batcharrayValidated.getValidatedInstrument().clear();
				currrowcount = 0;
			}
			batcharrayValidated.getValidatedInstrument()
					.add(instrument.getInstruments().getValidatedInstrument()
							.get(i));
			currrowcount++;
		}

		// Run last batch extraction
		if (batcharrayValidated.getValidatedInstrument().size() > 0) {
			request.setInstruments(batcharrayValidated);
			try {
				response = proxy.extract(request, hch, null, hoih);
			} catch (SOAPFaultException e) {
				throw ApiUtility.ConvertFaultException(e);
			}

			System.out.println("Row Count = " + response.getRowCount());
			for (int j = 0; j < response.getRowCount(); j++) {
				for (int k = 0; k < response.getColumns().getExtractionColumn()
						.size(); k++) {
					if (response.getColumns().getExtractionColumn().get(k)
							.getValues().getDoubleOrEmptyOrDate().get(j) instanceof EmptyValue) {
						System.out.print(",");
					} else {
						System.out.print(response.getColumns()
								.getExtractionColumn().get(k).getValues()
								.getDoubleOrEmptyOrDate().get(j).toString()
								+ ",");
					}
				}
				System.out.println();
			}
		}
	}
}
