package samples.search.instrument;

import javax.xml.ws.Holder;
import javax.xml.ws.soap.SOAPFaultException;

import com.reuters.datascopeselect.extractionservice.v1.*;

import samples.ApiUtility;
import samples.ClientErrorException;
import samples.ServerErrorException;
import core.ISample;

public class FundsSample implements ISample {
	public void Execute() throws ClientErrorException, ServerErrorException,
			Exception {
		CredentialsHeader credentials = ApiUtility.CreateCredentials();
		Holder<CredentialsHeader> hch = ApiUtility.createhch(credentials);
		Holder<OperationInfoHeader> hoih = ApiUtility.createhoih();

		com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service service = new com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service();
		com.reuters.datascopeselect.extractionservice.v1.ExtractionService proxy = service
				.getExtractionService();

		String currencyType = "GBP"; // British Pound
		String domicileType = "VG"; // Virgin Islands (British)

		System.out.println("Searching for Funds Instruments that:");
		System.out.println("- use a currency type of (" + currencyType + ")");
		System.out.println("- have a domicile type of (" + domicileType + ")");

		InstrumentSearchRequestFunds request = new InstrumentSearchRequestFunds();
		request.setIdentifierType("RIC");
		request.setIdentifierSearchValue("*");
		request.setPreferredIdentifierType("RIC");
		request.setCurrencyType(currencyType);
		request.setDomicileType(domicileType);

		InstrumentSearchResponse response = new InstrumentSearchResponse();
		try {
			response = proxy.searchInstruments(request, hch, null, hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}
		System.out.println(Integer.toString(response.getInstruments()
				.getValidatedSearchInstrument().size())
				+ " Instrument found:");
		for (int i = 0; i < response.getInstruments()
				.getValidatedSearchInstrument().size(); i++) {
			System.out.println(response.getInstruments()
					.getValidatedSearchInstrument().get(i).getIdentifierType()
					+ ": "
					+ response.getInstruments().getValidatedSearchInstrument()
							.get(i).getIdentifierValue());
			System.out.println("  - Asset ID = "
					+ response.getInstruments().getValidatedSearchInstrument()
							.get(i).getAssetId());
			System.out.println("  - Quote ID = "
					+ response.getInstruments().getValidatedSearchInstrument()
							.get(i).getQuoteId());
			System.out.println("  - Source = "
					+ response.getInstruments().getValidatedSearchInstrument()
							.get(i).getSource());
			System.out.println("  - RIC = "
					+ response.getInstruments().getValidatedSearchInstrument()
							.get(i).getRic());
			System.out.println("  - File Code = "
					+ response.getInstruments().getValidatedSearchInstrument()
							.get(i).getFileCode());
			System.out.println("  - Object Type = "
					+ response.getInstruments().getValidatedSearchInstrument()
							.get(i).getObjectType());
			System.out.println("  - Segment = "
					+ response.getInstruments().getValidatedSearchInstrument()
							.get(i).getSegment());
			System.out.println("  - Description = "
					+ response.getInstruments().getValidatedSearchInstrument()
							.get(i).getDescription());

			if (response.getInstruments().getValidatedSearchInstrument().get(i)
					.getValidatedCounterparty() != null) {
				System.out.println("  - Counterparty = "
						+ response.getInstruments()
								.getValidatedSearchInstrument().get(i)
								.getValidatedCounterparty().getIdentifierType()
						+ ": "
						+ response.getInstruments()
								.getValidatedSearchInstrument().get(i)
								.getValidatedCounterparty()
								.getIdentifierValue());
				System.out.println("  - RCP ID = "
						+ response.getInstruments()
								.getValidatedSearchInstrument().get(i)
								.getValidatedCounterparty().getRcpId());
				System.out
						.println("  - Counterparty ID = "
								+ response.getInstruments()
										.getValidatedSearchInstrument().get(i)
										.getValidatedCounterparty()
										.getCounterpartyId());
				System.out.println("  - Description ID = "
						+ response.getInstruments()
								.getValidatedSearchInstrument().get(i)
								.getValidatedCounterparty().getDescription());
			}
		}
	}
}
