package samples.search.instrument;

import java.math.BigDecimal;
import java.util.GregorianCalendar;

import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.ws.Holder;
import javax.xml.ws.soap.SOAPFaultException;

import com.reuters.datascopeselect.extractionservice.v1.*;

import samples.ApiUtility;
import samples.ClientErrorException;
import samples.JavaUtility;
import samples.ServerErrorException;
import core.ISample;

public class GovCorpSample implements ISample {

	public void Execute() throws ClientErrorException, ServerErrorException,
			Exception {
		CredentialsHeader credentials = ApiUtility.CreateCredentials();
		Holder<CredentialsHeader> hch = ApiUtility.createhch(credentials);
		Holder<OperationInfoHeader> hoih = ApiUtility.createhoih();

		com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service service = new com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service();
		com.reuters.datascopeselect.extractionservice.v1.ExtractionService proxy = service
				.getExtractionService();

		String country = "US";
		String moodyType = "Aa2";
		String spType = "AA";
		String subGroup = "GBND";
		GregorianCalendar fromDate1 = new GregorianCalendar();
		fromDate1.set(GregorianCalendar.MONTH, 0);
		fromDate1.set(GregorianCalendar.DATE, 1);
		GregorianCalendar endDate1 = new GregorianCalendar();
		endDate1.set(GregorianCalendar.MONTH, 11);
		endDate1.set(GregorianCalendar.DATE, 31);
		XMLGregorianCalendar fromDate = JavaUtility
				.asXMLGregorianCalendar(JavaUtility.changedatetype(fromDate1));
		XMLGregorianCalendar toDate = JavaUtility
				.asXMLGregorianCalendar(JavaUtility.changedatetype(endDate1));
		BigDecimal couponLimit = new BigDecimal(5);

		System.out.println("Searching for Gov/Corp Instruments that:");
		System.out.println("- are government bonds");
		System.out.println("- have a sub-group type of 'Notes/Bonds Govt' ("
				+ subGroup + ")");
		System.out.println("- have a Moody's rating of " + moodyType);
		System.out.println("- have a S&P rating of " + spType);
		System.out.println("- are from the United States (" + country + ")");
		System.out
				.println("- mature this year ("
						+ JavaUtility.printCalendar(fromDate
								.toGregorianCalendar())
						+ " - "
						+ JavaUtility.printCalendar(toDate
								.toGregorianCalendar()) + ")");
		System.out.println("- have a coupon value that is greater than "
				+ couponLimit.toString());

		InstrumentSearchRequestGovCorp request = new InstrumentSearchRequestGovCorp();
		request.setIdentifierType("CSP");
		request.setIdentifierSearchValue("*");
		request.setPreferredIdentifierType("CSP");
		request.setIsGovernmentGroup(true);
		request.setSubGroupType(subGroup);
		ArrayOfString moodylist = new ArrayOfString();
		moodylist.getString().add(moodyType);
		ArrayOfString splist = new ArrayOfString();
		splist.getString().add(spType);
		request.setCountryType(country);
		request.setMaturityDateComparisonOperator("FROM");
		request.setMaturityDatePrimaryValue(fromDate);
		request.setMaturityDateSecondaryValue(toDate);
		request.setCouponComparisonOperator(">");
		request.setCouponPrimaryValue(couponLimit);

		InstrumentSearchResponse response = new InstrumentSearchResponse();
		try {
			response = proxy.searchInstruments(request, hch, null, hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}
		System.out.println(Integer.toString(response.getInstruments()
				.getValidatedSearchInstrument().size())
				+ " Instrument found:");
		for (int i = 0; i < response.getInstruments()
				.getValidatedSearchInstrument().size(); i++) {
			System.out.println(response.getInstruments()
					.getValidatedSearchInstrument().get(i).getIdentifierType()
					+ ": "
					+ response.getInstruments().getValidatedSearchInstrument()
							.get(i).getIdentifierValue());
			System.out.println("  - Asset ID = "
					+ response.getInstruments().getValidatedSearchInstrument()
							.get(i).getAssetId());
			System.out.println("  - Quote ID = "
					+ response.getInstruments().getValidatedSearchInstrument()
							.get(i).getQuoteId());
			System.out.println("  - Source = "
					+ response.getInstruments().getValidatedSearchInstrument()
							.get(i).getSource());
			System.out.println("  - RIC = "
					+ response.getInstruments().getValidatedSearchInstrument()
							.get(i).getRic());
			System.out.println("  - File Code = "
					+ response.getInstruments().getValidatedSearchInstrument()
							.get(i).getFileCode());
			System.out.println("  - Object Type = "
					+ response.getInstruments().getValidatedSearchInstrument()
							.get(i).getObjectType());
			System.out.println("  - Segment = "
					+ response.getInstruments().getValidatedSearchInstrument()
							.get(i).getSegment());
			System.out.println("  - Description = "
					+ response.getInstruments().getValidatedSearchInstrument()
							.get(i).getDescription());

			if (response.getInstruments().getValidatedSearchInstrument().get(i)
					.getValidatedCounterparty() != null) {
				System.out.println("  - Counterparty = "
						+ response.getInstruments()
								.getValidatedSearchInstrument().get(i)
								.getValidatedCounterparty().getIdentifierType()
						+ ": "
						+ response.getInstruments()
								.getValidatedSearchInstrument().get(i)
								.getValidatedCounterparty()
								.getIdentifierValue());
				System.out.println("  - RCP ID = "
						+ response.getInstruments()
								.getValidatedSearchInstrument().get(i)
								.getValidatedCounterparty().getRcpId());
				System.out
						.println("  - Counterparty ID = "
								+ response.getInstruments()
										.getValidatedSearchInstrument().get(i)
										.getValidatedCounterparty()
										.getCounterpartyId());
				System.out.println("  - Description ID = "
						+ response.getInstruments()
								.getValidatedSearchInstrument().get(i)
								.getValidatedCounterparty().getDescription());
			}
		}
	}
}
