package samples.usage;

import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.ws.Holder;
import javax.xml.ws.soap.SOAPFaultException;

import com.reuters.datascopeselect.extractionservice.v1.*;

import samples.ApiUtility;
import samples.ClientErrorException;
import samples.JavaUtility;
import samples.ServerErrorException;
import core.ISample;

public class RentalUsageInstrumentAsOfDateSummarySample implements ISample {

	public void Execute() throws ClientErrorException, ServerErrorException,
			Exception {
		CredentialsHeader credentials = ApiUtility.CreateCredentials();
		Holder<CredentialsHeader> hch = ApiUtility.createhch(credentials);
		Holder<OperationInfoHeader> hoih = ApiUtility.createhoih();

		com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service service = new com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service();
		com.reuters.datascopeselect.extractionservice.v1.ExtractionService proxy = service
				.getExtractionService();

		XMLGregorianCalendar asOfDateTime = JavaUtility
				.asXMLGregorianCalendar(JavaUtility.dateadd("MONTH", -1));

		System.out.println("Reporting usage for user:");
		System.out
				.println("- for rental instruments as of DateTime: "
						+ JavaUtility.printCalendar(asOfDateTime
								.toGregorianCalendar()));

		System.out.println("Extracting usage data ...");
		RentalUsageInstrumentAsOfDateSummaryRequest request = new RentalUsageInstrumentAsOfDateSummaryRequest();
		request.setAsOfDateTime(asOfDateTime);
		UsageResponse response = new UsageResponse();
		try {
			response = proxy.reportUsage(request, hch, null, hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}

		System.out.println("Row Count = " + response.getRowCount());
		System.out.println();
		System.out
				.println("Row Data (in CSV format - copy/paste into new file to open in a spreadsheet application) =");
		System.out.println();

		for (int i = 0; i < response.getColumns().getUsageColumn().size(); i++) {
			System.out.print(response.getColumns().getUsageColumn().get(i)
					.getName()
					+ ",");
		}
		System.out.println();

		for (int j = 0; j < response.getRowCount(); j++) {
			for (int i = 0; i < response.getColumns().getUsageColumn().size(); i++) {
				System.out.print(response.getColumns().getUsageColumn().get(i)
						.getValues().getAnyType().get(j).toString()
						+ ",");
			}
			System.out.println();
		}
	}
}
