package samples.validation;

import javax.xml.ws.Holder;
import javax.xml.ws.soap.SOAPFaultException;

import com.reuters.datascopeselect.extractionservice.v1.ArrayOfInstrumentIdentifier;
import com.reuters.datascopeselect.extractionservice.v1.CredentialsHeader;
import com.reuters.datascopeselect.extractionservice.v1.InstrumentIdentifier;
import com.reuters.datascopeselect.extractionservice.v1.InstrumentValidationRequest;
import com.reuters.datascopeselect.extractionservice.v1.InstrumentValidationResponse;
import com.reuters.datascopeselect.extractionservice.v1.OperationInfoHeader;

import samples.ApiUtility;
import samples.ClientErrorException;
import samples.ServerErrorException;
import core.ISample;

public class InstrumentSample implements ISample {

	public void Execute() throws ClientErrorException, ServerErrorException,
			Exception {
		CredentialsHeader credentials = ApiUtility.CreateCredentials();
		Holder<CredentialsHeader> hch = ApiUtility.createhch(credentials);
		Holder<OperationInfoHeader> hoih = ApiUtility.createhoih();

		com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service service = new com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service();
		com.reuters.datascopeselect.extractionservice.v1.ExtractionService proxy = service
				.getExtractionService();

		InstrumentIdentifier cf1 = new InstrumentIdentifier();
		cf1.setIdentifierType("RIC");
		cf1.setIdentifierValue("LCOc1");
		InstrumentIdentifier cf2 = new InstrumentIdentifier();
		cf2.setIdentifierType("RIC");
		cf2.setIdentifierValue("INVALID-VALUE");

		ArrayOfInstrumentIdentifier cf = new ArrayOfInstrumentIdentifier();
		cf.getInstrumentIdentifier().add(cf1);
		cf.getInstrumentIdentifier().add(cf2);

		System.out.println("Validating Instrument identifiers:");
		for (int m = 0; m < cf.getInstrumentIdentifier().size(); m++) {
			System.out.println(" "
					+ cf.getInstrumentIdentifier().get(m).getIdentifierType()
					+ ": "
					+ cf.getInstrumentIdentifier().get(m).getIdentifierValue());
		}
		InstrumentValidationResponse instrument = new InstrumentValidationResponse();
		InstrumentValidationRequest ivreq = new InstrumentValidationRequest();
		ivreq.setIdentifiers(cf);

		try {
			instrument = proxy.validateInstruments(ivreq, hch, null, hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}

		System.out.println("Validated Instruments:");
		for (int i = 0; i < instrument.getInstruments()
				.getValidatedInstrument().size(); i++) {
			System.out.println(instrument.getInstruments()
					.getValidatedInstrument().get(i).getIdentifierType()
					+ ": "
					+ instrument.getInstruments().getValidatedInstrument()
							.get(i).getIdentifierValue());
			System.out.println("  - Asset ID = "
					+ instrument.getInstruments().getValidatedInstrument()
							.get(i).getAssetId());
			System.out.println("  - Quote ID = "
					+ instrument.getInstruments().getValidatedInstrument()
							.get(i).getQuoteId());
			System.out.println("  - Source = "
					+ instrument.getInstruments().getValidatedInstrument()
							.get(i).getSource());
			System.out.println("  - RIC = "
					+ instrument.getInstruments().getValidatedInstrument()
							.get(i).getRic());
			System.out.println("  - File Code = "
					+ instrument.getInstruments().getValidatedInstrument()
							.get(i).getFileCode());
			System.out.println("  - Object Type = "
					+ instrument.getInstruments().getValidatedInstrument()
							.get(i).getObjectType());
			System.out.println("  - Segment = "
					+ instrument.getInstruments().getValidatedInstrument()
							.get(i).getSegment());
		}

		System.out.println("Unmapped identifiers:");
		for (int i = 0; i < instrument.getUnmappedIdentifiers()
				.getInstrumentIdentifier().size(); i++) {
			System.out.println(" "
					+ instrument.getUnmappedIdentifierErrors().getString()
							.get(i) + ":");
			System.out.println(instrument.getUnmappedIdentifiers()
					.getInstrumentIdentifier().get(i).getIdentifierType()
					+ ": "
					+ instrument.getUnmappedIdentifiers()
							.getInstrumentIdentifier().get(i)
							.getIdentifierValue());
		}
	}
}
