package samples.validation;

import javax.xml.ws.Holder;
import javax.xml.ws.soap.SOAPFaultException;

import com.reuters.datascopeselect.extractionservice.v1.*;

import samples.ApiUtility;
import samples.ClientErrorException;
import samples.ServerErrorException;
import core.ISample;

public class LegalEntitySample implements ISample {

	public void Execute() throws ClientErrorException, ServerErrorException,
			Exception {
		CredentialsHeader credentials = ApiUtility.CreateCredentials();
		Holder<CredentialsHeader> hch = ApiUtility.createhch(credentials);
		Holder<OperationInfoHeader> hoih = ApiUtility.createhoih();

		com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service service = new com.reuters.datascopeselect.extractionservice.v1.ExtractionService_Service();
		com.reuters.datascopeselect.extractionservice.v1.ExtractionService proxy = service
				.getExtractionService();

		LegalEntityIdentifier cpvalue1 = new LegalEntityIdentifier();
		cpvalue1.setIdentifierType("RCP");
		cpvalue1.setIdentifierValue("300012286");

		LegalEntityIdentifier cpvalue2 = new LegalEntityIdentifier();
		cpvalue2.setIdentifierType("RCP");
		cpvalue2.setIdentifierValue("INVALID-VALUE");

		ArrayOfLegalEntityIdentifier cpidentifiers = new ArrayOfLegalEntityIdentifier();
		cpidentifiers.getLegalEntityIdentifier().add(cpvalue1);
		cpidentifiers.getLegalEntityIdentifier().add(cpvalue2);

		System.out.println("Validating LegalEntity identifiers:");
		for (int m = 0; m < cpidentifiers.getLegalEntityIdentifier().size(); m++) {
			System.out.println(" "
					+ cpidentifiers.getLegalEntityIdentifier().get(m)
							.getIdentifierType()
					+ ": "
					+ cpidentifiers.getLegalEntityIdentifier().get(m)
							.getIdentifierValue());
		}

		LegalEntityValidationResponse validateresponse = new LegalEntityValidationResponse();
		LegalEntityValidationRequest checkleIdentifiers = new LegalEntityValidationRequest();
		checkleIdentifiers.setIdentifiers(cpidentifiers);

		try {
			validateresponse = proxy.validateLegalEntities(checkleIdentifiers,
					hch, null, hoih);
		} catch (SOAPFaultException e) {
			throw ApiUtility.ConvertFaultException(e);
		}

		for (int i = 0; i < validateresponse.getLegalEntities()
				.getValidatedLegalEntity().size(); i++) {
			System.out.println(validateresponse.getLegalEntities()
					.getValidatedLegalEntity().get(i).getIdentifierType()
					+ " : "
					+ validateresponse.getLegalEntities()
							.getValidatedLegalEntity().get(i)
							.getIdentifierValue());
			System.out.println("  - Entity ID = "
					+ validateresponse.getLegalEntities()
							.getValidatedLegalEntity().get(i).getEntityId());
			System.out.println("  - Org ID = "
					+ validateresponse.getLegalEntities()
							.getValidatedLegalEntity().get(i).getOrgId());
			System.out.println("  - Description = "
					+ validateresponse.getLegalEntities()
							.getValidatedLegalEntity().get(i).getDescription());
		}

		System.out.println("Unmapped identifiers:");
		for (int i = 0; i < validateresponse.getUnmappedIdentifiers()
				.getLegalEntityIdentifier().size(); i++) {
			System.out.println(validateresponse.getUnmappedIdentifiers()
					.getLegalEntityIdentifier().get(i).getIdentifierType()
					+ " : "
					+ validateresponse.getUnmappedIdentifiers()
							.getLegalEntityIdentifier().get(i)
							.getIdentifierValue());
		}
	}
}
